@@ -7,24 +7,15 @@ import collections
 import concurrent.futures
 import logging
 import math
+import multiprocessing as mp
 import pickle
 import statistics
 from functools import reduce
-from typing import (
-    Any,
-    Callable,
-    Collection,
-    Dict,
-    Iterable,
-    List,
-    Optional,
-    Sequence,
-    Set,
-    Tuple,
-)
+from typing import Any, Callable, Collection, Iterable, Sequence, Set
 
 import numpy as np
 import xgboost
+from imblearn.pipeline import Pipeline as ImblearnPipeline
 from imblearn.under_sampling import RandomUnderSampler
 from ortools.linear_solver import pywraplp
 from sklearn.compose import ColumnTransformer
@@ -41,12 +32,13 @@ from bugbug import (
 )
 from bugbug.model import Model
 
+logging.basicConfig(level=logging.INFO)
 logger = logging.getLogger(__name__)
 
 
 def get_commit_map(
-    revs: Optional[Set[test_scheduling.Revision]] = None,
-) -> Dict[test_scheduling.Revision, repository.CommitDict]:
+    revs: Set[test_scheduling.Revision] | None = None,
+) -> dict[test_scheduling.Revision, repository.CommitDict]:
     commit_map = {}
 
     for commit in repository.get_commits():
@@ -59,6 +51,342 @@ def get_commit_map(
     return commit_map
 
 
+def _get_cost(config: str) -> int:
+    costs = [
+        (("build", "opt"), 1),
+        (("build", "debug"), 2),
+        (("build", "plain"), 3),
+        (("linux2404-64", "opt"), 2),
+        (("linux2404-64", "debug"), 3),
+        (("linux1804-64", "opt"), 4),
+        (("linux1804-64", "debug"), 5),
+        (("linux2204-64", "opt"), 4),
+        (("linux2204-64", "debug"), 5),
+        (("windows11", "opt"), 6),
+        (("windows11", "debug"), 7),
+        (("windows10", "opt"), 8),
+        (("windows10", "debug"), 9),
+        (("android-em", "opt"), 10),
+        (("android-em", "debug"), 11),
+        (("mac", "opt"), 12),
+        (("mac", "debug"), 13),
+        (("asan", "opt"), 14),
+        (("asan", "debug"), 15),
+        (("linux1804-32", "opt"), 16),
+        (("linux1804-32", "debug"), 17),
+        (("android-hw", "opt"), 18),
+        (("android-hw", "debug"), 19),
+        (("tsan", "opt"), 20),
+        (("tsan", "debug"), 21),
+        (("test-linux1804-64-qr/opt-*",), 1),
+    ]
+
+    for substrings, cost in reversed(costs):
+        if all(s in config for s in substrings):
+            return cost
+
+    logger.warning(f"Couldn't find cost for {config}")
+    return max(cost for _, cost in costs)
+
+
+def _generate_equivalence_sets(
+    tasks: Iterable[str],
+    min_redundancy_confidence: float,
+    load_failing_together: Callable[[str], dict[str, tuple[float, float]]],
+    assume_redundant: bool,
+) -> list[Set[str]]:
+    # Generate 'equivalence sets', containing all tasks that are redundant with
+    # each other.
+    groups: list[Set[str]] = []
+    task_to_groups: dict[str, Set[int]] = collections.defaultdict(set)
+    incompatible_groups: dict[str, Set[int]] = collections.defaultdict(set)
+
+    def create_group(task: str) -> None:
+        if task in task_to_groups:
+            return
+
+        groups.append({task})
+        task_to_groups[task] = {len(groups) - 1}
+
+    # Add task1 to all equivalence groups where task2 is present, and likewise for task2.
+    # Skip groups which contain tasks that are not redundant with task1.
+    def add_to_groups(task1: str, task2: str) -> None:
+        found = False
+
+        if task1 in task_to_groups:
+            for i in task_to_groups[task1]:
+                if task2 in incompatible_groups and i in incompatible_groups[task2]:
+                    continue
+
+                groups[i].add(task2)
+                task_to_groups[task2].add(i)
+                found = True
+
+        if task2 in task_to_groups:
+            for i in task_to_groups[task2]:
+                if task1 in incompatible_groups and i in incompatible_groups[task1]:
+                    continue
+
+                groups[i].add(task1)
+                task_to_groups[task1].add(i)
+                found = True
+
+        # No suitable equivalence group was found for the tasks, create a new one.
+        if found:
+            return
+
+        group = {task1, task2}
+        groups.append(group)
+        task_to_groups[task1].add(len(groups) - 1)
+        task_to_groups[task2].add(len(groups) - 1)
+
+    def mark_incompatible(task1: str, task2: str) -> None:
+        if task1 in task_to_groups:
+            incompatible_groups[task2].update(task_to_groups[task1])
+
+        if task2 in task_to_groups:
+            incompatible_groups[task1].update(task_to_groups[task2])
+
+    sorted_tasks = sorted(tasks)
+    for i, task1 in enumerate(sorted_tasks):
+        create_group(task1)
+
+        try:
+            failing_together_stats = load_failing_together(task1)
+        except KeyError:
+            failing_together_stats = {}
+
+        for task2 in sorted_tasks[i + 1 :]:
+            try:
+                support, confidence = failing_together_stats[task2]
+            except KeyError:
+                if not assume_redundant:
+                    confidence = 0.0
+                else:
+                    confidence = 1.0
+
+            if confidence >= min_redundancy_confidence:
+                add_to_groups(task1, task2)
+            else:
+                mark_incompatible(task1, task2)
+
+    return groups
+
+
+def _get_equivalence_sets(min_redundancy_confidence: float):
+    try:
+        with open(f"equivalence_sets_{min_redundancy_confidence}.pickle", "rb") as fr:
+            return pickle.load(fr)
+    except FileNotFoundError:
+        past_failures_data = test_scheduling.PastFailures("group", True)
+
+        equivalence_sets = {}
+        failing_together = test_scheduling.get_failing_together_db("config_group", True)
+        all_configs = pickle.loads(failing_together[b"$ALL_CONFIGS$"])
+        configs_by_group = pickle.loads(failing_together[b"$CONFIGS_BY_GROUP$"])
+        for group in past_failures_data.all_runnables:
+            key = test_scheduling.failing_together_key(group)
+            try:
+                failing_together_stats = pickle.loads(failing_together[key])
+            except KeyError:
+                failing_together_stats = {}
+
+            def load_failing_together(
+                config: str,
+            ) -> dict[str, tuple[float, float]]:
+                return failing_together_stats[config]
+
+            configs = (
+                configs_by_group[group] if group in configs_by_group else all_configs
+            )
+
+            equivalence_sets[group] = _generate_equivalence_sets(
+                configs, min_redundancy_confidence, load_failing_together, True
+            )
+
+        with open(f"equivalence_sets_{min_redundancy_confidence}.pickle", "wb") as fw:
+            pickle.dump(equivalence_sets, fw)
+
+        return equivalence_sets
+
+
+def _solve_optimization(solver: pywraplp.Solver) -> bool:
+    # The MIP solver is usually fast (milliseconds). If we hit a weird problem,
+    # accept a suboptimal solution after 10 seconds.
+    solver.SetTimeLimit(10000)
+    status = solver.Solve()
+
+    if status == pywraplp.Solver.INFEASIBLE:
+        logger.warning("Optimization problem is infeasible")
+        return False
+    elif status == pywraplp.Solver.NOT_SOLVED:
+        logger.warning("Optimization problem could not be solved in time")
+        return False
+
+    return True
+
+
+def reduce_configs(
+    tasks: Collection[str],
+    min_redundancy_confidence: float,
+    assume_redundant: bool = False,
+) -> Set[str]:
+    failing_together = test_scheduling.get_failing_together_db("label", True)
+
+    def load_failing_together(task: str) -> dict[str, tuple[float, float]]:
+        key = test_scheduling.failing_together_key(task)
+        return pickle.loads(failing_together[key])
+
+    solver = pywraplp.Solver(
+        "select_configs", pywraplp.Solver.CBC_MIXED_INTEGER_PROGRAMMING
+    )
+
+    task_vars = {task: solver.BoolVar(task) for task in tasks}
+
+    equivalence_sets = _generate_equivalence_sets(
+        tasks, min_redundancy_confidence, load_failing_together, assume_redundant
+    )
+
+    # Create constraints to ensure at least one task from each set of equivalent
+    # sets is selected.
+
+    mutually_exclusive = True
+    seen = set()
+    for equivalence_set in equivalence_sets:
+        if any(config in seen for config in equivalence_set):
+            mutually_exclusive = False
+            break
+
+        seen |= equivalence_set
+
+    for equivalence_set in equivalence_sets:
+        sum_constraint = sum(task_vars[task] for task in equivalence_set)
+        if mutually_exclusive:
+            solver.Add(sum_constraint == 1)
+        else:
+            solver.Add(sum_constraint >= 1)
+
+    # Choose the best set of tasks that satisfy the constraints with the lowest cost.
+    solver.Minimize(sum(_get_cost(task) * task_vars[task] for task in task_vars.keys()))
+
+    if _solve_optimization(solver):
+        return {
+            task
+            for task, task_var in task_vars.items()
+            if task_var.solution_value() == 1
+        }
+    else:
+        return set(tasks)
+
+
+def select_configs(
+    groups: Collection[str],
+    min_redundancy_confidence: float,
+    max_configurations: int = 3,
+) -> dict[str, list[str]]:
+    failing_together = test_scheduling.get_failing_together_db("config_group", True)
+
+    all_configs = pickle.loads(failing_together[b"$ALL_CONFIGS$"])
+    all_configs_by_group = pickle.loads(failing_together[b"$CONFIGS_BY_GROUP$"])
+    config_costs = {config: _get_cost(config) for config in all_configs}
+
+    solver = pywraplp.Solver(
+        "select_configs", pywraplp.Solver.CBC_MIXED_INTEGER_PROGRAMMING
+    )
+
+    config_vars = {config: solver.BoolVar(config) for config in all_configs}
+    config_group_vars = {
+        (config, group): solver.BoolVar(f"{group}@{config}")
+        for group in groups
+        for config in (
+            all_configs_by_group[group]
+            if group in all_configs_by_group
+            else all_configs
+        )
+    }
+
+    equivalence_sets = _get_equivalence_sets(min_redundancy_confidence)
+
+    for group in groups:
+        # Create constraints to ensure at least one task from each set of equivalent
+        # groups is selected.
+
+        mutually_exclusive = True
+        seen = set()
+        for equivalence_set in equivalence_sets[group]:
+            if any(config in seen for config in equivalence_set):
+                mutually_exclusive = False
+                break
+
+            seen |= equivalence_set
+
+        set_variables = [
+            solver.BoolVar(f"{group}_{j}") for j in range(len(equivalence_sets[group]))
+        ]
+
+        for j, equivalence_set in enumerate(equivalence_sets[group]):
+            set_variable = set_variables[j]
+
+            sum_constraint = sum(
+                config_group_vars[(config, group)] for config in equivalence_set
+            )
+            if mutually_exclusive:
+                solver.Add(sum_constraint == set_variable)
+            else:
+                solver.Add(sum_constraint >= set_variable)
+
+        # Cap to max_configurations equivalence sets.
+        solver.Add(
+            sum(set_variables)
+            >= (
+                max_configurations
+                if len(set_variables) >= max_configurations
+                else len(set_variables)
+            )
+        )
+
+    for config in all_configs:
+        solver.Add(
+            sum(
+                config_group_var
+                for (c, g), config_group_var in config_group_vars.items()
+                if config == c
+            )
+            <= config_vars[config] * len(groups)
+        )
+
+    # Choose the best set of tasks that satisfy the constraints with the lowest cost.
+    # The cost is calculated as a sum of the following:
+    # - a fixed cost to use a config (since selecting a config has overhead, it is
+    #   wasteful to select a config only to run a single group);
+    # - a cost for each selected group.
+    # This way, for example, if we have a group that must run on a costly config and a
+    # group that can run either on the costly one or on a cheaper one, they'd both run
+    # on the costly one (since we have to pay its setup cost anyway).
+    solver.Minimize(
+        sum(10 * config_costs[c] * config_vars[c] for c in config_vars.keys())
+        + sum(
+            config_costs[config] * config_group_vars[(config, group)]
+            for config, group in config_group_vars.keys()
+        )
+    )
+
+    configs_by_group: dict[str, list[str]] = {}
+    for group in groups:
+        configs_by_group[group] = []
+
+    if _solve_optimization(solver):
+        for (config, group), config_group_var in config_group_vars.items():
+            if config_group_var.solution_value() == 1:
+                configs_by_group[group].append(config)
+    else:
+        least_cost_config = min(config_costs, key=lambda c: config_costs[c])
+        for group in groups:
+            configs_by_group[group].append(least_cost_config)
+
+    return configs_by_group
+
+
 class TestSelectModel(Model):
     def __init__(self, lemmatization=False, granularity="label", failures_skip=None):
         Model.__init__(self, lemmatization)
@@ -94,26 +422,25 @@ class TestSelectModel(Model):
             )
 
         self.cross_validation_enabled = False
+        self.calculate_importance = False
 
         self.entire_dataset_training = True
 
-        self.sampler = RandomUnderSampler(random_state=0)
-
         feature_extractors = [
-            test_scheduling_features.prev_failures(),
+            test_scheduling_features.PrevFailures(),
         ]
 
         if granularity == "label":
             feature_extractors += [
-                test_scheduling_features.platform(),
+                test_scheduling_features.Platform(),
                 # test_scheduling_features.chunk(),
-                test_scheduling_features.suite(),
+                test_scheduling_features.Suite(),
             ]
         elif granularity in ("group", "config_group"):
             feature_extractors += [
-                test_scheduling_features.path_distance(),
-                test_scheduling_features.common_path_components(),
-                test_scheduling_features.touched_together(),
+                test_scheduling_features.PathDistance(),
+                test_scheduling_features.CommonPathComponents(),
+                test_scheduling_features.TouchedTogether(),
             ]
 
         self.extraction_pipeline = Pipeline(
@@ -122,16 +449,23 @@ class TestSelectModel(Model):
                     "commit_extractor",
                     commit_features.CommitExtractor(feature_extractors, []),
                 ),
-                ("union", ColumnTransformer([("data", DictVectorizer(), "data")])),
             ]
         )
 
-        self.clf = xgboost.XGBClassifier(n_jobs=utils.get_physical_cpu_count())
-        self.clf.set_params(predictor="cpu_predictor")
+        self.clf = ImblearnPipeline(
+            [
+                ("union", ColumnTransformer([("data", DictVectorizer(), "data")])),
+                ("sampler", RandomUnderSampler(random_state=0)),
+                (
+                    "estimator",
+                    xgboost.XGBClassifier(n_jobs=utils.get_physical_cpu_count()),
+                ),
+            ]
+        )
 
     def get_pushes(
         self, apply_filters: bool = False
-    ) -> Tuple[List[Dict[str, Any]], int]:
+    ) -> tuple[list[dict[str, Any]], int]:
         pushes = []
         for revs, test_datas in test_scheduling.get_test_scheduling_history(
             self.granularity
@@ -172,8 +506,10 @@ class TestSelectModel(Model):
             len(push["failures"]) + len(push["passes"])
             for push in pushes[:train_push_len]
         )
-        print(
-            f"{train_push_len} pushes in the training set (corresponding to {train_len} push/jobs)"
+        logger.info(
+            "%d pushes in the training set (corresponding to %d push/jobs)",
+            train_push_len,
+            train_len,
         )
         return X[:train_len], X[train_len:], y[:train_len], y[train_len:]
 
@@ -209,21 +545,17 @@ class TestSelectModel(Model):
             for name in push["passes"]:
                 classes[(push["revs"][0], name)] = 0
 
-        print("{} pushes considered".format(len(pushes)))
-        print(
-            "{} pushes with at least one failure".format(
-                sum(1 for push in pushes if len(push["failures"]) > 0)
-            )
+        logger.info("%d pushes considered", len(pushes))
+        logger.info(
+            "%d pushes with at least one failure",
+            sum(len(push["failures"]) > 0 for push in pushes),
         )
-        print(
-            "{} push/jobs failed".format(
-                sum(1 for label in classes.values() if label == 1)
-            )
+        logger.info(
+            "%d push/jobs failed", sum(label == 1 for label in classes.values())
         )
-        print(
-            "{} push/jobs did not fail".format(
-                sum(1 for label in classes.values() if label == 0)
-            )
+        logger.info(
+            "%d push/jobs did not fail",
+            sum(label == 0 for label in classes.values()),
         )
 
         return classes, [0, 1]
@@ -232,15 +564,14 @@ class TestSelectModel(Model):
         self,
         commits: Sequence[repository.CommitDict],
         confidence: float = 0.5,
-        push_num: Optional[int] = None,
-    ) -> Dict[str, float]:
+        push_num: int | None = None,
+    ) -> dict[str, float]:
         commit_data = commit_features.merge_commits(commits)
 
-        past_failures_data = test_scheduling.get_past_failures(self.granularity, True)
+        past_failures_data = test_scheduling.PastFailures(self.granularity, False)
 
         if push_num is None:
-            push_num = past_failures_data["push_num"] + 1
-        all_runnables = past_failures_data["all_runnables"]
+            push_num = past_failures_data.push_num + 1
 
         commit_tests = []
         for data in test_scheduling.generate_data(
@@ -248,7 +579,7 @@ class TestSelectModel(Model):
             past_failures_data,
             commit_data,
             push_num,
-            all_runnables,
+            past_failures_data.all_runnables,
             tuple(),
             tuple(),
         ):
@@ -263,350 +594,6 @@ class TestSelectModel(Model):
             for i in selected_indexes
         }
 
-    def _get_cost(self, config: str) -> int:
-        costs = [
-            (("build", "opt"), 1),
-            (("build", "debug"), 2),
-            (("build", "plain"), 3),
-            (("linux1804-64", "opt"), 2),
-            (("linux1804-64", "debug"), 3),
-            (("windows10", "opt"), 4),
-            (("windows10", "debug"), 5),
-            (("android-em", "opt"), 6),
-            (("android-em", "debug"), 7),
-            (("windows7", "opt"), 8),
-            (("windows7", "debug"), 9),
-            (("mac", "opt"), 10),
-            (("mac", "debug"), 11),
-            (("asan", "opt"), 12),
-            (("asan", "debug"), 13),
-            (("linux1804-32", "opt"), 14),
-            (("linux1804-32", "debug"), 15),
-            (("android-hw", "opt"), 16),
-            (("android-hw", "debug"), 17),
-            (("tsan", "opt"), 18),
-            (("tsan", "debug"), 19),
-            (("test-linux1804-64/opt-*-e10s",), 1),
-        ]
-
-        for substrings, cost in reversed(costs):
-            if all(s in config for s in substrings):
-                return cost
-
-        raise Exception(f"Couldn't find cost for {config}")
-
-    def _generate_equivalence_sets(
-        self,
-        tasks: Iterable[str],
-        min_redundancy_confidence: float,
-        load_failing_together: Callable[[str], Dict[str, Tuple[float, float]]],
-        assume_redundant: bool,
-    ) -> List[Set[str]]:
-        # Generate 'equivalence sets', containing all tasks that are redundant with
-        # each other.
-        groups: List[Set[str]] = []
-        task_to_groups: Dict[str, Set[int]] = collections.defaultdict(set)
-        incompatible_groups: Dict[str, Set[int]] = collections.defaultdict(set)
-
-        def create_group(task: str) -> None:
-            if task in task_to_groups:
-                return
-
-            groups.append({task})
-            task_to_groups[task] = {len(groups) - 1}
-
-        # Add task1 to all equivalence groups where task2 is present, and likewise for task2.
-        # Skip groups which contain tasks that are not redundant with task1.
-        def add_to_groups(task1: str, task2: str) -> None:
-            found = False
-
-            if task1 in task_to_groups:
-                for i in task_to_groups[task1]:
-                    if task2 in incompatible_groups and i in incompatible_groups[task2]:
-                        continue
-
-                    groups[i].add(task2)
-                    task_to_groups[task2].add(i)
-                    found = True
-
-            if task2 in task_to_groups:
-                for i in task_to_groups[task2]:
-                    if task1 in incompatible_groups and i in incompatible_groups[task1]:
-                        continue
-
-                    groups[i].add(task1)
-                    task_to_groups[task1].add(i)
-                    found = True
-
-            # No suitable equivalence group was found for the tasks, create a new one.
-            if found:
-                return
-
-            group = {task1, task2}
-            groups.append(group)
-            task_to_groups[task1].add(len(groups) - 1)
-            task_to_groups[task2].add(len(groups) - 1)
-
-        def mark_incompatible(task1: str, task2: str) -> None:
-            if task1 in task_to_groups:
-                incompatible_groups[task2].update(task_to_groups[task1])
-
-            if task2 in task_to_groups:
-                incompatible_groups[task1].update(task_to_groups[task2])
-
-        sorted_tasks = sorted(tasks)
-        for i, task1 in enumerate(sorted_tasks):
-            create_group(task1)
-
-            try:
-                failing_together_stats = load_failing_together(task1)
-            except KeyError:
-                failing_together_stats = {}
-
-            for task2 in sorted_tasks[i + 1 :]:
-                try:
-                    support, confidence = failing_together_stats[task2]
-                except KeyError:
-                    if not assume_redundant:
-                        confidence = 0.0
-                    else:
-                        confidence = 1.0
-
-                if confidence >= min_redundancy_confidence:
-                    add_to_groups(task1, task2)
-                else:
-                    mark_incompatible(task1, task2)
-
-        return groups
-
-    def _get_equivalence_sets(self, min_redundancy_confidence: float):
-        try:
-            with open(
-                f"equivalence_sets_{min_redundancy_confidence}.pickle", "rb"
-            ) as f:
-                return pickle.load(f)
-        except FileNotFoundError:
-            past_failures_data = test_scheduling.get_past_failures(
-                self.granularity, True
-            )
-            all_runnables = past_failures_data["all_runnables"]
-
-            equivalence_sets = {}
-            failing_together = test_scheduling.get_failing_together_db(
-                "config_group", True
-            )
-            all_configs = pickle.loads(failing_together[b"$ALL_CONFIGS$"])
-            configs_by_group = pickle.loads(failing_together[b"$CONFIGS_BY_GROUP$"])
-            for group in all_runnables:
-                key = test_scheduling.failing_together_key(group)
-                try:
-                    failing_together_stats = pickle.loads(failing_together[key])
-                except KeyError:
-                    failing_together_stats = {}
-
-                def load_failing_together(
-                    config: str,
-                ) -> Dict[str, Tuple[float, float]]:
-                    return failing_together_stats[config]
-
-                configs = (
-                    configs_by_group[group]
-                    if group in configs_by_group
-                    else all_configs
-                )
-
-                equivalence_sets[group] = self._generate_equivalence_sets(
-                    configs, min_redundancy_confidence, load_failing_together, True
-                )
-
-            with open(
-                f"equivalence_sets_{min_redundancy_confidence}.pickle", "wb"
-            ) as f:
-                pickle.dump(equivalence_sets, f)
-
-            return equivalence_sets
-
-    def _solve_optimization(self, solver: pywraplp.Solver) -> bool:
-        # The MIP solver is usually fast (milliseconds). If we hit a weird problem,
-        # accept a suboptimal solution after 10 seconds.
-        solver.SetTimeLimit(10000)
-        status = solver.Solve()
-
-        if status == pywraplp.Solver.INFEASIBLE:
-            logger.warning("Optimization problem is infeasible")
-            return False
-        elif status == pywraplp.Solver.NOT_SOLVED:
-            logger.warning("Optimization problem could not be solved in time")
-            return False
-
-        return True
-
-    def reduce(
-        self,
-        tasks: Collection[str],
-        min_redundancy_confidence: float,
-        assume_redundant: bool = False,
-    ) -> Set[str]:
-        failing_together = test_scheduling.get_failing_together_db(
-            self.granularity, True
-        )
-
-        def load_failing_together(task: str) -> Dict[str, Tuple[float, float]]:
-            key = test_scheduling.failing_together_key(task)
-            return pickle.loads(failing_together[key])
-
-        solver = pywraplp.Solver(
-            "select_configs", pywraplp.Solver.CBC_MIXED_INTEGER_PROGRAMMING
-        )
-
-        task_vars = {task: solver.BoolVar(task) for task in tasks}
-
-        equivalence_sets = self._generate_equivalence_sets(
-            tasks, min_redundancy_confidence, load_failing_together, assume_redundant
-        )
-
-        # Create constraints to ensure at least one task from each set of equivalent
-        # sets is selected.
-
-        mutually_exclusive = True
-        seen = set()
-        for equivalence_set in equivalence_sets:
-            if any(config in seen for config in equivalence_set):
-                mutually_exclusive = False
-                break
-
-            seen |= equivalence_set
-
-        for equivalence_set in equivalence_sets:
-            sum_constraint = sum(task_vars[task] for task in equivalence_set)
-            if mutually_exclusive:
-                solver.Add(sum_constraint == 1)
-            else:
-                solver.Add(sum_constraint >= 1)
-
-        # Choose the best set of tasks that satisfy the constraints with the lowest cost.
-        solver.Minimize(
-            sum(self._get_cost(task) * task_vars[task] for task in task_vars.keys())
-        )
-
-        if self._solve_optimization(solver):
-            return {
-                task
-                for task, task_var in task_vars.items()
-                if task_var.solution_value() == 1
-            }
-        else:
-            return set(tasks)
-
-    def select_configs(
-        self,
-        groups: Collection[str],
-        min_redundancy_confidence: float,
-        max_configurations: int = 3,
-    ) -> Dict[str, List[str]]:
-        failing_together = test_scheduling.get_failing_together_db("config_group", True)
-
-        all_configs = pickle.loads(failing_together[b"$ALL_CONFIGS$"])
-        all_configs_by_group = pickle.loads(failing_together[b"$CONFIGS_BY_GROUP$"])
-        config_costs = {config: self._get_cost(config) for config in all_configs}
-
-        solver = pywraplp.Solver(
-            "select_configs", pywraplp.Solver.CBC_MIXED_INTEGER_PROGRAMMING
-        )
-
-        config_vars = {config: solver.BoolVar(config) for config in all_configs}
-        config_group_vars = {
-            (config, group): solver.BoolVar(f"{group}@{config}")
-            for group in groups
-            for config in (
-                all_configs_by_group[group]
-                if group in all_configs_by_group
-                else all_configs
-            )
-        }
-
-        equivalence_sets = self._get_equivalence_sets(min_redundancy_confidence)
-
-        for group in groups:
-            # Create constraints to ensure at least one task from each set of equivalent
-            # groups is selected.
-
-            mutually_exclusive = True
-            seen = set()
-            for equivalence_set in equivalence_sets[group]:
-                if any(config in seen for config in equivalence_set):
-                    mutually_exclusive = False
-                    break
-
-                seen |= equivalence_set
-
-            set_variables = [
-                solver.BoolVar(f"{group}_{j}")
-                for j in range(len(equivalence_sets[group]))
-            ]
-
-            for j, equivalence_set in enumerate(equivalence_sets[group]):
-                set_variable = set_variables[j]
-
-                sum_constraint = sum(
-                    config_group_vars[(config, group)] for config in equivalence_set
-                )
-                if mutually_exclusive:
-                    solver.Add(sum_constraint == set_variable)
-                else:
-                    solver.Add(sum_constraint >= set_variable)
-
-            # Cap to max_configurations equivalence sets.
-            solver.Add(
-                sum(set_variables)
-                >= (
-                    max_configurations
-                    if len(set_variables) >= max_configurations
-                    else len(set_variables)
-                )
-            )
-
-        for config in all_configs:
-            solver.Add(
-                sum(
-                    config_group_var
-                    for (c, g), config_group_var in config_group_vars.items()
-                    if config == c
-                )
-                <= config_vars[config] * len(groups)
-            )
-
-        # Choose the best set of tasks that satisfy the constraints with the lowest cost.
-        # The cost is calculated as a sum of the following:
-        # - a fixed cost to use a config (since selecting a config has overhead, it is
-        #   wasteful to select a config only to run a single group);
-        # - a cost for each selected group.
-        # This way, for example, if we have a group that must run on a costly config and a
-        # group that can run either on the costly one or on a cheaper one, they'd both run
-        # on the costly one (since we have to pay its setup cost anyway).
-        solver.Minimize(
-            sum(10 * config_costs[c] * config_vars[c] for c in config_vars.keys())
-            + sum(
-                config_costs[config] * config_group_vars[(config, group)]
-                for config, group in config_group_vars.keys()
-            )
-        )
-
-        configs_by_group: Dict[str, List[str]] = {}
-        for group in groups:
-            configs_by_group[group] = []
-
-        if self._solve_optimization(solver):
-            for (config, group), config_group_var in config_group_vars.items():
-                if config_group_var.solution_value() == 1:
-                    configs_by_group[group].append(config)
-        else:
-            least_cost_config = min(config_costs, key=lambda c: config_costs[c])
-            for group in groups:
-                configs_by_group[group].append(least_cost_config)
-
-        return configs_by_group
-
     def evaluation(self) -> None:
         # Get a test set of pushes on which to test the model.
         pushes, train_push_len = self.get_pushes(False)
@@ -614,7 +601,7 @@ class TestSelectModel(Model):
         # To evaluate the model with reductions enabled, we need to regenerate the failing together DB, using
         # only failure data from the training pushes (otherwise, we'd leak training information into the test
         # set).
-        print("Generate failing together DB (restricted to training pushes)")
+        logger.info("Generate failing together DB (restricted to training pushes)")
         push_data_iter, push_data_count, _ = test_scheduling.get_push_data(
             "label" if self.granularity == "label" else "config_group"
         )
@@ -663,20 +650,24 @@ class TestSelectModel(Model):
                 for push in test_pushes.values()
                 if "config_group_failures" not in push
             )
-            print(
-                f"{missing_config_group_failures} pushes without config_group failures"
+            logger.info(
+                "%d pushes without config_group failures", missing_config_group_failures
             )
 
-        print(
-            f"Testing on {len(test_pushes)} ({test_pushes_failures} with failures) out of {len(pushes)}. {len(all_tasks)} schedulable tasks."
+        logger.info(
+            "Testing on %d (%d with failures) out of %d. %d schedulable tasks.",
+            len(test_pushes),
+            test_pushes_failures,
+            len(pushes),
+            len(all_tasks),
         )
 
         del pushes
 
         commit_map = get_commit_map(all_revs)
 
-        past_failures_data = test_scheduling.get_past_failures(self.granularity, True)
-        last_push_num = past_failures_data["push_num"]
+        past_failures_data = test_scheduling.PastFailures(self.granularity, True)
+        last_push_num = past_failures_data.push_num
         past_failures_data.close()
 
         # Select tests for all the pushes in the test set.
@@ -703,16 +694,16 @@ class TestSelectModel(Model):
         def do_eval(
             executor: concurrent.futures.ProcessPoolExecutor,
             confidence_threshold: float,
-            reduction: Optional[float],
-            cap: Optional[int],
-            minimum: Optional[int],
+            reduction: float | None,
+            cap: int | None,
+            minimum: int | None,
         ) -> None:
-            futures: Dict[concurrent.futures.Future, Dict[str, Any]] = {}
+            futures: dict[concurrent.futures.Future, dict[str, Any]] = {}
             for push in test_pushes.values():
                 futures[
                     executor.submit(
                         eval_apply_transforms,
-                        self,
+                        self.granularity,
                         push,
                         confidence_threshold,
                         reduction,
@@ -724,10 +715,8 @@ class TestSelectModel(Model):
             for future in concurrent.futures.as_completed(futures):
                 exc = future.exception()
                 if exc is not None:
-                    print(
-                        "Exception {} while running {}".format(
-                            exc, futures[future]["revs"][0]
-                        )
+                    logger.error(
+                        "Exception %s while running %s", exc, futures[future]["revs"][0]
                     )
                     for f in futures:
                         f.cancel()
@@ -842,22 +831,33 @@ class TestSelectModel(Model):
                 )
                 percentage_caught_one_config_group = (
                     100
-                    * num_caught_one_config_group
-                    / num_failing_pushes_with_config_group
+                    * (
+                        num_caught_one_config_group
+                        / num_failing_pushes_with_config_group
+                    )
+                    if num_failing_pushes_with_config_group > 0
+                    else 1
                 )
-                average_caught_percentage_config_group = 100 * statistics.mean(
-                    result["caught_percentage_config_group"]
-                    for result in test_pushes.values()
-                    if "caught_percentage_config_group" in result
-                    and result["caught_percentage_config_group"] is not None
+                average_caught_percentage_config_group = (
+                    100
+                    * statistics.mean(
+                        result["caught_percentage_config_group"]
+                        for result in test_pushes.values()
+                        if "caught_percentage_config_group" in result
+                        and result["caught_percentage_config_group"] is not None
+                    )
+                    if num_failing_pushes_with_config_group > 0
+                    else 1
                 )
 
                 message += f" In {percentage_caught_one_config_group}% of pushes we caught at least one config/group failure. On average, we caught {average_caught_percentage_config_group}% of all seen config/group failures."
 
-            print(message)
+            logger.info(message)
 
         with concurrent.futures.ProcessPoolExecutor(
-            max_workers=utils.get_physical_cpu_count()
+            max_workers=utils.get_physical_cpu_count(),
+            # Fixing https://github.com/mozilla/bugbug/issues/3131
+            mp_context=mp.get_context("fork"),
         ) as executor:
             scenarios = [
                 (None, None, None),
@@ -870,13 +870,13 @@ class TestSelectModel(Model):
                 # Pre-generate equivalence sets, so when we run the config selection in multiple processes
                 # we don't risk concurrent writes to the equivalence sets file.
                 if reduction is not None and self.granularity == "group":
-                    self._get_equivalence_sets(reduction)
+                    _get_equivalence_sets(reduction)
 
                 for confidence_threshold in [0.5, 0.7, 0.8, 0.85, 0.9, 0.95]:
                     do_eval(executor, confidence_threshold, reduction, cap, minimum)
 
     def get_feature_names(self):
-        return self.extraction_pipeline.named_steps["union"].get_feature_names()
+        return self.clf.named_steps["union"].get_feature_names_out()
 
 
 class TestLabelSelectModel(TestSelectModel):
@@ -894,7 +894,9 @@ class TestConfigGroupSelectModel(TestSelectModel):
         TestSelectModel.__init__(self, lemmatization, "config_group")
 
 
-def eval_apply_transforms(model, push, confidence_threshold, reduction, cap, minimum):
+def eval_apply_transforms(
+    granularity, push, confidence_threshold, reduction, cap, minimum
+):
     group_configs = None
 
     selected = set(
@@ -904,10 +906,10 @@ def eval_apply_transforms(model, push, confidence_threshold, reduction, cap, min
     )
 
     if reduction is not None:
-        if model.granularity == "label":
-            selected = model.reduce(selected, reduction)
-        elif model.granularity == "group":
-            group_configs = model.select_configs(selected, reduction)
+        if granularity == "label":
+            selected = reduce_configs(selected, reduction)
+        elif granularity == "group":
+            group_configs = select_configs(selected, reduction)
 
     if minimum is not None and len(selected) < minimum:
         remaining = [
