@@ -17,38 +17,34 @@ from typing import (
     Any,
     Callable,
     Deque,
-    Dict,
     Generator,
     Iterable,
     Iterator,
-    List,
     NewType,
-    Optional,
     Set,
-    Tuple,
     Union,
     cast,
 )
 
-import requests
 from tqdm import tqdm
 
 from bugbug import db, repository
-from bugbug.utils import ExpQueue, LMDBDict
+from bugbug.utils import ExpQueue, LMDBDict, get_session, get_user_agent
 
+logging.basicConfig(level=logging.INFO)
 logger = logging.getLogger(__name__)
 
 Revision = NewType("Revision", str)
 Task = NewType("Task", str)
 Group = NewType("Group", str)
-ConfigGroup = NewType("ConfigGroup", Tuple[str, Group])
+ConfigGroup = NewType("ConfigGroup", tuple[str, Group])
 Runnable = Union[Task, Group, ConfigGroup]
-PushResult = Tuple[
-    Tuple[Revision],
+PushResult = tuple[
+    tuple[Revision],
     Revision,
-    Tuple[Runnable, ...],
-    Tuple[Runnable, ...],
-    Tuple[Runnable, ...],
+    tuple[Runnable, ...],
+    tuple[Runnable, ...],
+    tuple[Runnable, ...],
 ]
 
 TEST_LABEL_SCHEDULING_DB = "data/test_label_scheduling_history.pickle"
@@ -109,6 +105,7 @@ JOBS_TO_IGNORE = (
     "-awsy-",
     "-raptor-",
     "-talos-",
+    "perftest",
     "browsertime",
     "backlog",
     # inclusive test suites -- these *only* run when certain files have changed
@@ -117,14 +114,21 @@ JOBS_TO_IGNORE = (
     "jittest",
     "jsreftest",
     "android-hw-gfx",
+    "build-extensions-browser",
 )
 
 
+class UnexpectedGranularityError(ValueError):
+    def __init__(self, granularity):
+        message = f"Unexpected {granularity} granularity"
+        super().__init__(message)
+
+
 def filter_runnables(
-    runnables: Tuple[Runnable, ...], all_runnables: Set[Runnable], granularity: str
-) -> Tuple[Any, ...]:
+    runnables: tuple[Runnable, ...], all_runnables: Set[Runnable], granularity: str
+) -> tuple[Any, ...]:
     if granularity == "label":
-        tasks = cast(List[Task], runnables)
+        tasks = cast(list[Task], runnables)
         return tuple(
             task
             for task in tasks
@@ -167,16 +171,16 @@ def rename_task(task: str) -> str:
 
 # Handle "meaningless" labeling changes ("meaningless" as they shouldn't really affect test scheduling).
 def rename_runnables(
-    granularity: str, runnables: Tuple[Runnable, ...]
-) -> Tuple[Runnable, ...]:
+    granularity: str, runnables: tuple[Runnable, ...]
+) -> tuple[Runnable, ...]:
     if granularity == "label":
-        tasks = cast(List[Task], runnables)
+        tasks = cast(list[Task], runnables)
         return tuple(Task(rename_task(task)) for task in tasks)
     elif granularity == "group":
-        groups = cast(List[Group], runnables)
+        groups = cast(list[Group], runnables)
         return tuple(Group(group.split(":")[0]) for group in groups)
     elif granularity == "config_group":
-        config_groups = cast(List[ConfigGroup], runnables)
+        config_groups = cast(list[ConfigGroup], runnables)
         return tuple(
             ConfigGroup(
                 (
@@ -187,12 +191,12 @@ def rename_runnables(
             for config, group in config_groups
         )
     else:
-        raise Exception(f"Unexpected {granularity} granularity")
+        raise UnexpectedGranularityError(granularity)
 
 
 def get_push_data(
     granularity: str,
-) -> Tuple[Callable[[], Iterator[PushResult]], int, Tuple[Runnable, ...]]:
+) -> tuple[Callable[[], Iterator[PushResult]], int, tuple[Runnable, ...]]:
     if granularity == "label":
         push_data_db = PUSH_DATA_LABEL_DB
     elif granularity == "group":
@@ -210,7 +214,7 @@ def get_push_data(
 
         push_data_queue.append(elem)
 
-    logger.info(f"push data nodes: {push_data_count}")
+    logger.info("Push data nodes: %d", push_data_count)
 
     push_data = [
         (
@@ -238,7 +242,7 @@ def get_push_data(
             tuple(all_groups_set), cast(Set[Runnable], all_groups_set), "group"
         )
         all_groups_set = set(all_groups)
-        logger.info(f"{len(all_groups_set)} manifests run in the last 28 pushes")
+        logger.info("%d manifests run in the last 28 pushes", len(all_groups_set))
 
     all_runnables_set = set(
         sum((list(push_runnables) for _, _, push_runnables, _, _ in push_data), [])
@@ -248,7 +252,7 @@ def get_push_data(
         tuple(all_runnables_set), all_runnables_set, granularity
     )
     all_runnables_set = set(all_runnables)
-    logger.info(f"{len(all_runnables_set)} runnables run in the last 28 pushes")
+    logger.info("%d runnables run in the last 28 pushes", len(all_runnables_set))
 
     def push_data_iter() -> Iterator[PushResult]:
         return (
@@ -284,8 +288,9 @@ def get_push_data(
             )
         )
 
-        print(
-            f"{manifest_combinations} possible combinations of manifests on configurations"
+        logger.info(
+            "%d possible combinations of manifests on configurations",
+            manifest_combinations,
         )
 
     return push_data_iter, push_data_count, all_runnables
@@ -299,27 +304,68 @@ def get_test_scheduling_history(granularity):
     elif granularity == "config_group":
         test_scheduling_db = TEST_CONFIG_GROUP_SCHEDULING_DB
     else:
-        raise Exception(f"{granularity} granularity unsupported")
+        raise UnexpectedGranularityError(granularity)
 
     for obj in db.read(test_scheduling_db):
         yield obj["revs"], obj["data"]
 
 
-def get_past_failures(granularity, readonly):
-    if granularity == "label":
-        past_failures_db = os.path.join("data", PAST_FAILURES_LABEL_DB)
-    elif granularity == "group":
-        past_failures_db = os.path.join("data", PAST_FAILURES_GROUP_DB)
-    elif granularity == "config_group":
-        past_failures_db = os.path.join("data", PAST_FAILURES_CONFIG_GROUP_DB)
-    else:
-        raise Exception(f"{granularity} granularity unsupported")
+class PastFailures:
+    def __init__(self, granularity, readonly):
+        if granularity == "label":
+            past_failures_db = os.path.join("data", PAST_FAILURES_LABEL_DB)
+        elif granularity == "group":
+            past_failures_db = os.path.join("data", PAST_FAILURES_GROUP_DB)
+        elif granularity == "config_group":
+            assert False, "config_group granularity not supported for past failures"
+        else:
+            raise UnexpectedGranularityError(granularity)
+        self.granularity = granularity
 
-    return shelve.Shelf(
-        LMDBDict(past_failures_db[: -len(".tar.zst")], readonly=readonly),
-        protocol=pickle.DEFAULT_PROTOCOL,
-        writeback=not readonly,
-    )
+        self.db = shelve.Shelf(
+            LMDBDict(past_failures_db[: -len(".tar.zst")], readonly=readonly),
+            protocol=pickle.DEFAULT_PROTOCOL,
+            writeback=not readonly,
+        )
+
+    @property
+    def push_num(self) -> int:
+        return self.db["push_num"]
+
+    @push_num.setter
+    def push_num(self, value: int) -> None:
+        self.db["push_num"] = value
+
+    @property
+    def all_runnables(self):
+        return self.db["all_runnables"]
+
+    @all_runnables.setter
+    def all_runnables(self, value) -> None:
+        self.db["all_runnables"] = value
+
+    def get(self, key: str) -> ExpQueue | None:
+        value = self.db.get(key, None)
+
+        # Fallback on INI if the group is now TOML.
+        if value is None and self.granularity == "group" and key.endswith(".toml"):
+            ini_key = f"{key[:-4]}ini"
+            try:
+                value = self.db[ini_key]
+                self.db[key] = value
+            except KeyError:
+                return None
+
+        return value
+
+    def set(self, key: str, value: ExpQueue) -> None:
+        self.db[key] = value
+
+    def sync(self) -> None:
+        self.db.sync()
+
+    def close(self) -> None:
+        self.db.close()
 
 
 def get_failing_together_db_path(granularity: str) -> str:
@@ -328,7 +374,7 @@ def get_failing_together_db_path(granularity: str) -> str:
     elif granularity == "config_group":
         path = FAILING_TOGETHER_CONFIG_GROUP_DB
     else:
-        raise Exception(f"{granularity} granularity unsupported")
+        raise UnexpectedGranularityError(granularity)
 
     return os.path.join("data", path[: -len(".tar.zst")])
 
@@ -358,9 +404,9 @@ def remove_failing_together_db(granularity: str) -> None:
 
 def close_failing_together_db(granularity: str) -> None:
     global failing_together
-    assert (
-        granularity in failing_together
-    ), f"Failing together probabilities DB for {granularity} granularity was not open"
+    assert granularity in failing_together, (
+        f"Failing together probabilities DB for {granularity} granularity was not open"
+    )
     failing_together[granularity].close()
     failing_together.pop(granularity)
 
@@ -369,7 +415,7 @@ def generate_failing_together_probabilities(
     granularity: str,
     push_data: Iterator[PushResult],
     push_data_count: int,
-    up_to: str = None,
+    up_to: str | None = None,
 ) -> None:
     # TODO: we should consider the probabilities of `task1 failure -> task2 failure` and
     # `task2 failure -> task1 failure` separately, as they could be different.
@@ -393,7 +439,7 @@ def generate_failing_together_probabilities(
                 count_single_failures[(task1, task2)] += 1
 
     all_available_configs: Set[str] = set()
-    available_configs_by_group: Dict[Group, Set[str]] = collections.defaultdict(set)
+    available_configs_by_group: dict[Group, Set[str]] = collections.defaultdict(set)
 
     for (
         revisions,
@@ -450,7 +496,7 @@ def generate_failing_together_probabilities(
 
         stats[couple] = (support, confidence)
 
-    logger.info(f"{skipped} couples skipped because their support was too low")
+    logger.info("%d couples skipped because their support was too low", skipped)
 
     logger.info("Redundancies with the highest support and confidence:")
     for couple, (support, confidence) in sorted(
@@ -459,7 +505,13 @@ def generate_failing_together_probabilities(
         failure_count = count_both_failures[couple]
         run_count = count_runs[couple]
         logger.info(
-            f"{couple[0]} - {couple[1]} redundancy confidence {confidence}, support {support} ({failure_count} over {run_count})."
+            "%s - %s redundancy confidence %f, support %d (%d over %d).",
+            couple[0],
+            couple[1],
+            confidence,
+            support,
+            failure_count,
+            run_count,
         )
 
     logger.info("Redundancies with the highest confidence and lowest support:")
@@ -469,7 +521,13 @@ def generate_failing_together_probabilities(
         failure_count = count_both_failures[couple]
         run_count = count_runs[couple]
         logger.info(
-            f"{couple[0]} - {couple[1]} redundancy confidence {confidence}, support {support} ({failure_count} over {run_count})."
+            "%s - %s redundancy confidence %f, support %d (%d over %d).",
+            couple[0],
+            couple[1],
+            confidence,
+            support,
+            failure_count,
+            run_count,
         )
 
     failing_together: dict = {}
@@ -518,7 +576,7 @@ def generate_failing_together_probabilities(
             failing_together[couple[0]][couple[1]] = (support, confidence)
 
     for percentage, count in count_redundancies.most_common():
-        logger.info(f"{count} with {percentage} confidence")
+        logger.info("%d with %s confidence", count, percentage)
 
     failing_together_db = get_failing_together_db(granularity, False)
 
@@ -588,7 +646,7 @@ def set_touched_together(f1: str, f2: str) -> None:
         )
 
 
-def update_touched_together() -> Generator[None, Optional[Revision], None]:
+def update_touched_together() -> Generator[None, Revision | None, None]:
     touched_together = get_touched_together_db(False)
     last_analyzed = (
         touched_together[b"last_analyzed"]
@@ -653,15 +711,14 @@ def _read_and_update_past_failures(
     for item in items:
         full_key = key + item
 
-        is_new = full_key not in past_failures
+        cur = past_failures.get(full_key)
+        is_new = cur is None
 
         if is_new:
             if not is_regression:
                 continue
 
             cur = ExpQueue(round(push_num / 100), int(HISTORICAL_TIMESPAN / 100) + 1, 0)
-        else:
-            cur = past_failures[full_key]
 
         value = cur[round(push_num / 100)]
 
@@ -673,7 +730,7 @@ def _read_and_update_past_failures(
         if is_regression:
             cur[round(push_num / 100)] = value + 1
             if is_new:
-                past_failures[full_key] = cur
+                past_failures.set(full_key, cur)
 
     return (
         sum(values_total),
@@ -685,7 +742,7 @@ def _read_and_update_past_failures(
 
 def generate_data(
     granularity: str,
-    past_failures: int,
+    past_failures: PastFailures,
     commit: repository.CommitDict,
     push_num: int,
     runnables: Iterable[str],
@@ -816,22 +873,25 @@ def generate_data(
         yield obj
 
 
-def get_failure_bugs(since: datetime, until: datetime) -> List[Dict[str, int]]:
-    r = requests.get(
+def get_failure_bugs(since: datetime, until: datetime) -> list[dict[str, int]]:
+    r = get_session("treeherder").get(
         "https://treeherder.mozilla.org/api/failures/?startday={}&endday={}&tree=trunk".format(
             since.strftime("%Y-%m-%d"), until.strftime("%Y-%m-%d")
         ),
-        headers={"Accept": "application/json", "User-Agent": "bugbug"},
+        headers={"Accept": "application/json", "User-Agent": get_user_agent()},
     )
     r.raise_for_status()
     return r.json()
 
 
-def get_test_info(date: datetime) -> Dict[str, Any]:
-    r = requests.get(
+def get_test_info(date: datetime) -> dict[str, Any]:
+    r = get_session("firefox-ci-tc").get(
         "https://firefox-ci-tc.services.mozilla.com/api/index/v1/task/gecko.v2.mozilla-central.pushdate.{}.latest.source.test-info-all/artifacts/public/test-info-all-tests.json".format(
             date.strftime("%Y.%m.%d")
-        )
+        ),
+        headers={
+            "User-Agent": get_user_agent(),
+        },
     )
     r.raise_for_status()
     return r.json()
