@@ -17,10 +17,13 @@ from collections import deque
 from contextlib import contextmanager
 from datetime import datetime
 from functools import lru_cache
-from typing import Any, Iterator, List, Optional
+from importlib.metadata import PackageNotFoundError
+from typing import Any, Iterator
 
 import boto3
+import botocore
 import dateutil.parser
+import libmozdata
 import lmdb
 import numpy as np
 import psutil
@@ -28,7 +31,6 @@ import requests
 import scipy
 import taskcluster
 import zstandard
-from pkg_resources import DistributionNotFound
 from requests.packages.urllib3.util.retry import Retry
 from sklearn.base import BaseEstimator, TransformerMixin
 from sklearn.compose import ColumnTransformer
@@ -71,15 +73,15 @@ def to_array(val):
 
 
 class StructuredColumnTransformer(ColumnTransformer):
-    def _hstack(self, Xs):
-        result = super()._hstack(Xs)
+    def _hstack(self, Xs, n_samples):
+        result = super()._hstack(Xs, n_samples=n_samples)
 
         transformer_names = (name for name, transformer, column in self.transformers_)
         types = []
         for i, (f, transformer_name) in enumerate(zip(Xs, transformer_names)):
             types.append((transformer_name, result.dtype, (f.shape[1],)))
 
-        return result.todense().view(np.dtype(types))
+        return result.view(np.dtype(types))
 
 
 class DictExtractor(BaseEstimator, TransformerMixin):
@@ -93,9 +95,20 @@ class DictExtractor(BaseEstimator, TransformerMixin):
         return np.array([elem[self.key] for elem in data]).reshape(-1, 1)
 
 
+class MergeText(BaseEstimator, TransformerMixin):
+    def __init__(self, cols):
+        self.cols = cols
+
+    def fit(self, X, y=None):
+        return self
+
+    def transform(self, X):
+        return X[self.cols].apply(lambda row: " ".join(row), axis=1)
+
+
 class MissingOrdinalEncoder(OrdinalEncoder):
-    """
-    Ordinal encoder that ignores missing values encountered after training.
+    """Ordinal encoder that ignores missing values encountered after training.
+
     Workaround for issue: scikit-learn/scikit-learn#11997
     """
 
@@ -110,9 +123,9 @@ class MissingOrdinalEncoder(OrdinalEncoder):
 
 
 def get_taskcluster_options() -> dict:
-    """
-    Helper to get the Taskcluster setup options
-    according to current environment (local or Taskcluster)
+    """Get the Taskcluster setup options according to current environment.
+
+    The current environment could be local.
     """
     options = taskcluster.optionsFromEnvironment()
     proxy_url = os.environ.get("TASKCLUSTER_PROXY_URL")
@@ -128,8 +141,8 @@ def get_taskcluster_options() -> dict:
     return options
 
 
-def get_secret(secret_id: str) -> Any:
-    """Return the secret value"""
+def get_secret(secret_id: str, default_value: str | None = None) -> Any:
+    """Return the secret value."""
     env_variable_name = f"BUGBUG_{secret_id}"
 
     # Try in the environment first
@@ -145,7 +158,10 @@ def get_secret(secret_id: str) -> Any:
         secrets = taskcluster.Secrets(get_taskcluster_options())
         secret_bucket = secrets.get(tc_secret_id)
 
-        return secret_bucket["secret"][secret_id]
+        return secret_bucket["secret"].get(secret_id, default_value)
+
+    elif default_value is not None:
+        return default_value
 
     else:
         raise ValueError("Failed to find secret {}".format(secret_id))
@@ -173,8 +189,29 @@ def upload_s3(paths: str) -> None:
         transfer.upload_file(path, "communitytc-bugbug", path)
 
 
+def exists_s3(path: str) -> bool:
+    credentials = get_s3_credentials()
+
+    client = boto3.client(
+        "s3",
+        aws_access_key_id=credentials["accessKeyId"],
+        aws_secret_access_key=credentials["secretAccessKey"],
+        aws_session_token=credentials["sessionToken"],
+    )
+
+    try:
+        client.head_object(Bucket="communitytc-bugbug", Key=path)
+        return True
+    except botocore.exceptions.ClientError as e:
+        if e.response["Error"]["Code"] == "404":
+            return False
+        raise
+
+
 def download_check_etag(url, path=None):
-    r = requests.head(url, allow_redirects=True)
+    session = get_session(urllib.parse.urlparse(url).netloc)
+    r = session.head(url, allow_redirects=True)
+    r.raise_for_status()
 
     if path is None:
         path = url.split("/")[-1]
@@ -190,21 +227,35 @@ def download_check_etag(url, path=None):
     if old_etag == new_etag:
         return False
 
-    r = requests.get(url, stream=True)
+    r = session.get(
+        url,
+        stream=True,
+        headers={
+            "User-Agent": get_user_agent(),
+        },
+    )
     r.raise_for_status()
 
     with open(path, "wb") as f:
         for chunk in r.iter_content(chunk_size=1048576):
             f.write(chunk)
 
+    r.raise_for_status()
+
     with open(f"{path}.etag", "w") as f:
         f.write(new_etag)
 
     return True
 
 
-def get_last_modified(url: str) -> Optional[datetime]:
-    r = requests.head(url, allow_redirects=True)
+def get_last_modified(url: str) -> datetime | None:
+    session = get_session(urllib.parse.urlparse(url).netloc)
+    r = session.head(url, allow_redirects=True)
+
+    if r.status_code == 404:
+        return None
+
+    r.raise_for_status()
 
     if "Last-Modified" not in r.headers:
         return None
@@ -217,32 +268,50 @@ def download_model(model_name: str) -> str:
     if not version:
         try:
             version = f"v{get_bugbug_version()}"
-        except DistributionNotFound:
+        except PackageNotFoundError:
             version = "latest"
 
     path = f"{model_name}model"
-    url = f"https://community-tc.services.mozilla.com/api/index/v1/task/project.bugbug.train_{model_name}.{version}/artifacts/public/{path}.zst"
-    logger.info(f"Downloading {url}...")
+    url = f"https://community-tc.services.mozilla.com/api/index/v1/task/project.bugbug.train_{model_name}.{version}/artifacts/public/{path}.tar.zst"
+    logger.info("Downloading %s...", url)
     updated = download_check_etag(url)
     if updated:
-        zstd_decompress(path)
-        os.remove(f"{path}.zst")
-    assert os.path.exists(path), "Decompressed file exists"
+        extract_tar_zst(f"{path}.tar.zst")
+        os.remove(f"{path}.tar.zst")
+    assert os.path.exists(path), "Decompressed directory exists"
     return path
 
 
 def zstd_compress(path: str) -> None:
-    cctx = zstandard.ZstdCompressor(threads=-1)
-    with open(path, "rb") as input_f:
-        with open(f"{path}.zst", "wb") as output_f:
-            cctx.copy_stream(input_f, output_f)
+    if not os.path.exists(path):
+        raise FileNotFoundError(errno.ENOENT, os.strerror(errno.ENOENT), path)
+
+    try:
+        subprocess.run(["zstdmt", "-f", path], check=True)
+    except FileNotFoundError as error:
+        logger.warning(
+            "%s. Falling back to zstandard API, which could be slower.", error
+        )
+        cctx = zstandard.ZstdCompressor()
+        with open(path, "rb") as input_f:
+            with open(f"{path}.zst", "wb") as output_f:
+                cctx.copy_stream(input_f, output_f)
 
 
 def zstd_decompress(path: str) -> None:
-    dctx = zstandard.ZstdDecompressor()
-    with open(f"{path}.zst", "rb") as input_f:
-        with open(path, "wb") as output_f:
-            dctx.copy_stream(input_f, output_f)
+    if not os.path.exists(f"{path}.zst"):
+        raise FileNotFoundError(errno.ENOENT, os.strerror(errno.ENOENT), path)
+
+    try:
+        subprocess.run(["zstdmt", "-df", f"{path}.zst"], check=True)
+    except FileNotFoundError as error:
+        logger.warning(
+            "%s. Falling back to zstandard API, which could be slower.", error
+        )
+        dctx = zstandard.ZstdDecompressor()
+        with open(f"{path}.zst", "rb") as input_f:
+            with open(path, "wb") as output_f:
+                dctx.copy_stream(input_f, output_f)
 
 
 @contextmanager
@@ -270,14 +339,28 @@ def create_tar_zst(path: str) -> None:
     if not os.path.exists(inner_path):
         raise FileNotFoundError(errno.ENOENT, os.strerror(errno.ENOENT), inner_path)
 
-    subprocess.run(["tar", "-I", "zstd", "-cf", path, inner_path], check=True)
+    try:
+        subprocess.run(["tar", "-I", "zstdmt", "-cf", path, inner_path], check=True)
+    except subprocess.CalledProcessError as error:
+        logger.warning(
+            "%s. Falling back to zstandard API, which could be slower.", error
+        )
+        with open_tar_zst(path, "w") as tar:
+            tar.add(inner_path)
 
 
 def extract_tar_zst(path: str) -> None:
     if not os.path.exists(path):
         raise FileNotFoundError(errno.ENOENT, os.strerror(errno.ENOENT), path)
 
-    subprocess.run(["tar", "-I", "zstd", "-xf", path], check=True)
+    try:
+        subprocess.run(["tar", "-I", "zstdmt", "-xf", path], check=True)
+    except subprocess.CalledProcessError as error:
+        logger.warning(
+            "%s. Falling back to zstandard API, which could be slower.", error
+        )
+        with open_tar_zst(path, "r") as tar:
+            tar.extractall()
 
 
 def extract_file(path: str) -> None:
@@ -292,15 +375,15 @@ def extract_file(path: str) -> None:
 
 
 class CustomJsonEncoder(json.JSONEncoder):
-    """A custom Json Encoder to support Numpy types"""
+    """A custom Json Encoder to support Numpy types."""
 
     def default(self, obj):
         try:
-            return np.asscalar(obj)
+            return obj.item()
         except (ValueError, IndexError, AttributeError, TypeError):
             pass
 
-        return super().default(self, obj)
+        return super().default(obj)
 
 
 class ExpQueue:
@@ -325,9 +408,9 @@ class ExpQueue:
         return self.start_day + (self.list.maxlen - 1)
 
     def __getitem__(self, day: int) -> Any:
-        assert (
-            day >= self.start_day
-        ), f"Can't get a day ({day}) from earlier than start day ({self.start_day})"
+        assert day >= self.start_day, (
+            f"Can't get a day ({day}) from earlier than start day ({self.start_day})"
+        )
 
         if day < 0:
             return self.default
@@ -389,6 +472,11 @@ class LMDBDict:
     def __setitem__(self, key: bytes, value: Any) -> None:
         self.txn.put(key, value, dupdata=False)
 
+    def keys(self):
+        cursor = self.txn.cursor()
+        for key, value in cursor:
+            yield key.tobytes()
+
 
 def get_free_tcp_port() -> int:
     tcp = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
@@ -437,10 +525,24 @@ def get_session(name: str) -> requests.Session:
     return session
 
 
-def get_hgmo_stack(branch: str, revision: str) -> List[bytes]:
-    """Load descriptions of patches in the stack for a given revision"""
+def get_user_agent():
+    return get_secret("USER_AGENT", "bugbug")
+
+
+def setup_libmozdata():
+    libmozdata.config.set_config(libmozdata.config.ConfigEnv())
+    os.environ["LIBMOZDATA_CFG_USER-AGENT_NAME"] = get_user_agent()
+
+
+def get_hgmo_stack(branch: str, revision: str) -> list[bytes]:
+    """Load descriptions of patches in the stack for a given revision."""
     url = f"https://hg.mozilla.org/{branch}/json-automationrelevance/{revision}"
-    r = get_session("hgmo").get(url)
+    r = get_session("hgmo").get(
+        url,
+        headers={
+            "User-Agent": get_user_agent(),
+        },
+    )
     r.raise_for_status()
 
     def should_skip(changeset):
@@ -482,8 +584,8 @@ def extract_metadata(body: str) -> dict:
     return dict(match_list)
 
 
-def extract_private(issue_body: str) -> Optional[tuple]:
-    """Extract private issue information from public issue body
+def extract_private(issue_body: str) -> tuple | None:
+    """Extract private issue information from public issue body.
 
     Parse public issue body and extract private issue number and
     its owner/repository (webcompat repository usecase)
@@ -511,3 +613,13 @@ def escape_markdown(text: str) -> str:
         .replace(")", "\\)")
         .replace("|", "\\|")
     )
+
+
+def keep_as_is(x):
+    """A tokenizer that does nothing."""
+    return x
+
+
+def hg2git(hash):
+    r = requests.get(f"https://lando.moz.tools/api/hg2git/firefox/{hash}")
+    return r.json()["git_hash"]
