@@ -11,7 +11,7 @@ import re
 import subprocess
 from datetime import datetime
 from logging import INFO, basicConfig, getLogger
-from typing import Optional, Tuple, cast
+from typing import cast
 
 import dateutil.parser
 import hglib
@@ -26,7 +26,8 @@ from libmozdata.phabricator import PhabricatorAPI
 from scipy.stats import spearmanr
 
 from bugbug import db, repository, test_scheduling
-from bugbug.model import Model
+from bugbug.model import Model, get_transformer_pipeline
+from bugbug.models.regressor import RegressorModel
 from bugbug.models.testfailure import TestFailureModel
 from bugbug.utils import (
     download_check_etag,
@@ -63,11 +64,11 @@ REVIEWERS_RE = re.compile(
     r"([\s\(\.\[;,])"                   # before "r" delimiter
     + r"(" + SPECIFIER + r")"           # flag
     + r"("                              # capture all reviewers
-        + r"#?"                         # Optional "#" group reviewer prefix
+        + r"#?"                         # Optional "#" group reviewer prefix  # noqa: E131
         + IRC_NICK                      # reviewer
         + r"!?"                         # Optional "!" blocking indicator
         + r"(?:"                        # additional reviewers
-            + LIST                      # delimiter
+            + LIST                      # delimiter  # noqa: E131
             + r"(?![a-z0-9\.\-]+[=?])"  # don"t extend match into next flag
             + r"#?"                     # Optional "#" group reviewer prefix
             + IRC_NICK                  # reviewer
@@ -135,6 +136,8 @@ class CommitClassifier(object):
         method_defect_predictor_dir: str,
         use_single_process: bool,
         skip_feature_importance: bool,
+        phabricator_deployment: str | None = None,
+        diff_id: int | None = None,
     ):
         self.model_name = model_name
         self.repo_dir = repo_dir
@@ -145,9 +148,17 @@ class CommitClassifier(object):
         self.git_repo_dir = git_repo_dir
         if git_repo_dir:
             self.clone_git_repo(
-                "hg::https://hg.mozilla.org/mozilla-central", git_repo_dir
+                "hg::https://hg.mozilla.org/mozilla-unified", git_repo_dir
             )
 
+        self.revision = None
+        if diff_id is not None:
+            assert phabricator_deployment is not None
+            with hglib.open(self.repo_dir) as hg:
+                self.apply_phab(hg, phabricator_deployment, diff_id)
+
+                self.revision = hg.log(revrange="not public()")[0].node.decode("utf-8")
+
         self.method_defect_predictor_dir = method_defect_predictor_dir
         if method_defect_predictor_dir:
             self.clone_git_repo(
@@ -199,7 +210,7 @@ class CommitClassifier(object):
                 test_scheduling.TEST_LABEL_SCHEDULING_DB,
                 test_scheduling.PAST_FAILURES_LABEL_DB,
             )
-            self.past_failures_data = test_scheduling.get_past_failures("label", True)
+            self.past_failures_data = test_scheduling.PastFailures("label", True)
 
             self.testfailure_model = cast(
                 TestFailureModel, TestFailureModel.load(download_model("testfailure"))
@@ -207,12 +218,12 @@ class CommitClassifier(object):
             assert self.testfailure_model is not None
 
     def clone_git_repo(self, repo_url, repo_dir, rev="origin/branches/default/tip"):
-        logger.info(f"Cloning {repo_url}...")
+        logger.info("Cloning %s...", repo_url)
 
         if not os.path.exists(repo_dir):
             tenacity.retry(
-                wait=tenacity.wait_exponential(multiplier=1, min=16, max=64),
-                stop=tenacity.stop_after_attempt(5),
+                wait=tenacity.wait_exponential(multiplier=2, min=2),
+                stop=tenacity.stop_after_attempt(7),
             )(
                 lambda: subprocess.run(
                     ["git", "clone", "--quiet", repo_url, repo_dir], check=True
@@ -220,8 +231,8 @@ class CommitClassifier(object):
             )()
 
         tenacity.retry(
-            wait=tenacity.wait_exponential(multiplier=1, min=16, max=64),
-            stop=tenacity.stop_after_attempt(5),
+            wait=tenacity.wait_exponential(multiplier=2, min=2),
+            stop=tenacity.stop_after_attempt(7),
         )(
             lambda: subprocess.run(
                 ["git", "fetch"],
@@ -236,7 +247,9 @@ class CommitClassifier(object):
         )
 
     def update_commit_db(self):
-        repository.clone(self.repo_dir, update=True)
+        repository.clone(
+            self.repo_dir, "https://hg.mozilla.org/mozilla-unified", update=True
+        )
 
         assert db.download(repository.COMMITS_DB, support_files_too=True)
 
@@ -281,7 +294,7 @@ class CommitClassifier(object):
             # Stop as soon as a base revision is available
             if self.has_revision(hg, patch.base_revision):
                 logger.info(
-                    f"Stopping at diff {patch.id} and revision {patch.base_revision}"
+                    "Stopping at diff %s and revision %s", patch.id, patch.base_revision
                 )
                 break
 
@@ -302,13 +315,13 @@ class CommitClassifier(object):
         hg_base = needed_stack[0].base_revision
         if not self.has_revision(hg, hg_base):
             logger.warning("Missing base revision {} from Phabricator".format(hg_base))
-            hg_base = "tip"
+            hg_base = "default"
 
         if hg_base:
             hg.update(rev=hg_base, clean=True)
-            logger.info(f"Updated repo to {hg_base}")
+            logger.info("Updated repo to %s", hg_base)
 
-            if self.git_repo_dir and hg_base != "tip":
+            if self.git_repo_dir and hg_base != "default":
                 try:
                     self.git_base = tuple(
                         vcs_map.mercurial_to_git(self.git_repo_dir, [hg_base])
@@ -318,18 +331,20 @@ class CommitClassifier(object):
                         check=True,
                         cwd=self.git_repo_dir,
                     )
-                    logger.info(f"Updated git repo to {self.git_base}")
+                    logger.info("Updated git repo to %s", self.git_base)
                 except Exception as e:
-                    logger.info(f"Updating git repo to Mercurial {hg_base} failed: {e}")
+                    logger.info(
+                        "Updating git repo to Mercurial %s failed: %s", hg_base, e
+                    )
 
         def load_user(phid):
             if phid.startswith("PHID-USER"):
                 return phabricator_api.load_user(user_phid=phid)
             elif phid.startswith("PHID-PROJ"):
                 # TODO: Support group reviewers somehow.
-                logger.info(f"Skipping group reviewer {phid}")
+                logger.info("Skipping group reviewer %s", phid)
             else:
-                raise Exception(f"Unsupported reviewer {phid}")
+                raise ValueError(f"Unsupported reviewer {phid}")
 
         for patch in needed_stack:
             revision = revisions[patch.phid]
@@ -368,7 +383,7 @@ class CommitClassifier(object):
                 message = replace_reviewers(message, reviewers)
 
             logger.info(
-                f"Applying {patch.phid} from revision {revision['id']}: {message}"
+                "Applying %s from revision %s: %s", patch.phid, revision["id"], message
             )
 
             hg.import_(
@@ -377,32 +392,20 @@ class CommitClassifier(object):
                 user=f"{author_name} <{author_email}>".encode("utf-8"),
             )
 
-            if self.git_repo_dir:
-                patch_proc = subprocess.Popen(
-                    ["patch", "-p1", "--no-backup-if-mismatch", "--force"],
-                    stdin=subprocess.PIPE,
-                    cwd=self.git_repo_dir,
-                )
-                patch_proc.communicate(patch.patch.encode("utf-8"))
-                assert patch_proc.returncode == 0, "Failed to apply patch"
+        latest_rev = repository.get_revs(hg, -1)[0]
 
-                subprocess.run(
-                    [
-                        "git",
-                        "-c",
-                        f"user.name={author_name}",
-                        "-c",
-                        f"user.email={author_email}",
-                        "commit",
-                        "-am",
-                        message,
-                    ],
-                    check=True,
-                    cwd=self.git_repo_dir,
-                )
+        if self.git_repo_dir:
+            subprocess.run(
+                ["git", "cinnabar", "fetch", f"hg::{self.repo_dir}", latest_rev],
+                check=True,
+                cwd=self.git_repo_dir,
+            )
 
     def generate_feature_importance_data(self, probs, importance):
-        X_shap_values = shap.TreeExplainer(self.model.clf).shap_values(self.X)
+        _X = get_transformer_pipeline(self.clf).transform(self.X)
+        X_shap_values = shap.TreeExplainer(
+            self.clf.named_steps["estimator"]
+        ).shap_values(_X)
 
         pred_class = self.model.le.inverse_transform([probs[0].argmax()])[0]
 
@@ -414,8 +417,8 @@ class CommitClassifier(object):
             value = importance["importances"]["values"][0, int(feature_index)]
 
             shap.summary_plot(
-                X_shap_values[:, int(feature_index)].reshape(self.X.shape[0], 1),
-                self.X[:, int(feature_index)].reshape(self.X.shape[0], 1),
+                X_shap_values[:, int(feature_index)].reshape(_X.shape[0], 1),
+                _X[:, int(feature_index)].reshape(_X.shape[0], 1),
                 feature_names=[""],
                 plot_type="layered_violin",
                 show=False,
@@ -427,7 +430,7 @@ class CommitClassifier(object):
             img.seek(0)
             base64_img = base64.b64encode(img.read()).decode("ascii")
 
-            X = self.X[:, int(feature_index)]
+            X = _X[:, int(feature_index)]
             y = self.y[X != 0]
             X = X[X != 0]
             spearman = spearmanr(X, y)
@@ -453,32 +456,37 @@ class CommitClassifier(object):
 
             logger.info("Feature: {}".format(name))
             logger.info("Shap value: {}{}".format("+" if (is_positive) else "-", val))
-            logger.info(f"spearman:  {spearman}")
-            logger.info(f"value: {value}")
-            logger.info(f"overall mean: {np.mean(X)}")
-            logger.info(f"overall median: {np.median(X)}")
-            logger.info(f"mean for y == 0: {np.mean(clean_X)}")
-            logger.info(f"mean for y == 1: {np.mean(buggy_X)}")
-            logger.info(f"median for y == 0: {np.median(clean_X)}")
-            logger.info(f"median for y == 1: {np.median(buggy_X)}")
+            logger.info("spearman: %f", spearman)
+            logger.info("value: %f", value)
+            logger.info("overall mean: %f", np.mean(X))
+            logger.info("overall median: %f", np.median(X))
+            logger.info("mean for y == 0: %f", np.mean(clean_X))
+            logger.info("mean for y == 1: %f", np.mean(buggy_X))
+            logger.info("median for y == 0: %f", np.median(clean_X))
+            logger.info("median for y == 1: %f", np.median(buggy_X))
+
             logger.info(
-                f"perc_buggy_values_higher_than_median: {perc_buggy_values_higher_than_median}"
+                "perc_buggy_values_higher_than_median: %f",
+                perc_buggy_values_higher_than_median,
             )
             logger.info(
-                f"perc_buggy_values_lower_than_median: {perc_buggy_values_lower_than_median}"
+                "perc_buggy_values_lower_than_median: %f",
+                perc_buggy_values_lower_than_median,
             )
             logger.info(
-                f"perc_clean_values_higher_than_median: {perc_clean_values_higher_than_median}"
+                "perc_clean_values_higher_than_median: %f",
+                perc_clean_values_higher_than_median,
             )
             logger.info(
-                f"perc_clean_values_lower_than_median: {perc_clean_values_lower_than_median}"
+                "perc_clean_values_lower_than_median: %f",
+                perc_clean_values_lower_than_median,
             )
 
             features.append(
                 {
                     "index": i + 1,
                     "name": name,
-                    "shap": float(f'{"+" if (is_positive) else "-"}{val}'),
+                    "shap": float(f"{'+' if (is_positive) else '-'}{val}"),
                     "value": importance["importances"]["values"][0, int(feature_index)],
                     "spearman": spearman,
                     "median": median,
@@ -574,26 +582,14 @@ class CommitClassifier(object):
 
     def classify(
         self,
-        revision: Optional[str] = None,
-        phabricator_deployment: Optional[str] = None,
-        diff_id: Optional[int] = None,
-        runnable_jobs_path: Optional[str] = None,
+        revision: str | None = None,
+        runnable_jobs_path: str | None = None,
     ) -> None:
-        if revision is not None:
-            assert phabricator_deployment is None
-            assert diff_id is None
-
-        if diff_id is not None:
-            assert phabricator_deployment is not None
-            assert revision is None
-
         self.update_commit_db()
 
-        if phabricator_deployment is not None and diff_id is not None:
-            with hglib.open(self.repo_dir) as hg:
-                self.apply_phab(hg, phabricator_deployment, diff_id)
-
-                revision = hg.log(revrange="not public()")[0].node.decode("utf-8")
+        if self.revision is not None:
+            assert revision is None
+            revision = self.revision
 
             commits = repository.download_commits(
                 self.repo_dir,
@@ -625,7 +621,7 @@ class CommitClassifier(object):
         else:
             self.classify_test_select(commits, runnable_jobs_path)
 
-    def classify_regressor(self, commits: Tuple[repository.CommitDict, ...]) -> None:
+    def classify_regressor(self, commits: tuple[repository.CommitDict, ...]) -> None:
         # We use "clean" (or "dirty") commits as the background dataset for feature importance.
         # This way, we can see the features which are most important in differentiating
         # the current commit from the "clean" (or "dirty") commits.
@@ -640,8 +636,14 @@ class CommitClassifier(object):
         if not self.skip_feature_importance:
             self.generate_feature_importance_data(probs, importance)
 
-        with open("probs.json", "w") as f:
-            json.dump(probs[0].tolist(), f)
+        results = {
+            "probs": probs[0].tolist(),
+        }
+        if self.model_name == "regressor":
+            results["risk_band"] = RegressorModel.find_risk_band(probs[0][1])
+
+        with open("results.json", "w") as f:
+            json.dump(results, f)
 
         if self.model_name == "regressor" and self.method_defect_predictor_dir:
             self.classify_methods(commits[-1])
@@ -651,7 +653,7 @@ class CommitClassifier(object):
             commits[-1], probabilities=True
         )
 
-        logger.info(f"Test failure risk: {testfailure_probs[0][1]}")
+        logger.info("Test failure risk: %f", testfailure_probs[0][1])
 
         if not runnable_jobs_path:
             runnable_jobs = {}
@@ -669,7 +671,7 @@ class CommitClassifier(object):
         selected_tasks = list(
             self.model.select_tests(
                 commits, float(get_secret("TEST_SELECTION_CONFIDENCE_THRESHOLD"))
-            ).values()
+            ).keys()
         )
 
         # XXX: For now, only restrict to linux64 test tasks (as for runnable jobs above, we could remove these right away).
@@ -686,8 +688,8 @@ class CommitClassifier(object):
             )
 
         # This should be kept in sync with the test scheduling history retriever script.
+        cleaned_selected_tasks = []
         if len(runnable_jobs) > 0:
-            cleaned_selected_tasks = []
             for selected_task in selected_tasks:
                 if (
                     selected_task.startswith("test-linux64")
@@ -841,6 +843,14 @@ def main() -> None:
 
     args = parser.parse_args()
 
+    if args.revision is not None:
+        assert args.phabricator_deployment is None
+        assert args.diff_id is None
+
+    if args.diff_id is not None:
+        assert args.phabricator_deployment is not None
+        assert args.revision is None
+
     classifier = CommitClassifier(
         args.model,
         args.repo_dir,
@@ -848,10 +858,10 @@ def main() -> None:
         args.method_defect_predictor_dir,
         args.use_single_process,
         args.skip_feature_importance,
+        args.phabricator_deployment,
+        args.diff_id,
     )
-    classifier.classify(
-        args.revision, args.phabricator_deployment, args.diff_id, args.runnable_jobs
-    )
+    classifier.classify(args.revision, args.runnable_jobs)
 
 
 if __name__ == "__main__":
