@@ -4,11 +4,9 @@
 # You can obtain one at http://mozilla.org/MPL/2.0/.
 
 import argparse
-import concurrent.futures
 import os
 import subprocess
 import threading
-import time
 from collections import defaultdict
 from datetime import datetime
 from logging import INFO, basicConfig, getLogger
@@ -64,14 +62,16 @@ class RegressorFinder(object):
 
         with ThreadPoolExecutorResult(max_workers=3) as executor:
             if self.git_repo_url is not None:
-                logger.info(f"Cloning {self.git_repo_url} to {self.git_repo_dir}...")
+                logger.info("Cloning %s to %s...", self.git_repo_url, self.git_repo_dir)
                 executor.submit(
                     self.clone_git_repo, self.git_repo_url, self.git_repo_dir
                 )
 
             if self.tokenized_git_repo_url is not None:
                 logger.info(
-                    f"Cloning {self.tokenized_git_repo_url} to {self.tokenized_git_repo_dir}..."
+                    "Cloning %s to %s...",
+                    self.tokenized_git_repo_url,
+                    self.tokenized_git_repo_dir,
                 )
                 executor.submit(
                     self.clone_git_repo,
@@ -85,21 +85,21 @@ class RegressorFinder(object):
     def clone_git_repo(self, repo_url, repo_dir):
         if not os.path.exists(repo_dir):
             tenacity.retry(
-                wait=tenacity.wait_exponential(multiplier=1, min=16, max=64),
-                stop=tenacity.stop_after_attempt(5),
+                wait=tenacity.wait_exponential(multiplier=2, min=2),
+                stop=tenacity.stop_after_attempt(7),
             )(
                 lambda: subprocess.run(
                     ["git", "clone", "--quiet", repo_url, repo_dir], check=True
                 )
             )()
 
-            logger.info(f"{repo_dir} cloned")
+            logger.info("%s cloned", repo_dir)
 
-        logger.info(f"Fetching {repo_dir}")
+        logger.info("Fetching %s", repo_dir)
 
         tenacity.retry(
-            wait=tenacity.wait_exponential(multiplier=1, min=16, max=64),
-            stop=tenacity.stop_after_attempt(5),
+            wait=tenacity.wait_exponential(multiplier=2, min=2),
+            stop=tenacity.stop_after_attempt(7),
         )(
             lambda: subprocess.run(
                 ["git", "fetch", "--quiet"],
@@ -109,7 +109,7 @@ class RegressorFinder(object):
             )
         )()
 
-        logger.info(f"{repo_dir} fetched")
+        logger.info("%s fetched", repo_dir)
 
     def init_mapping(self):
         if self.tokenized_git_repo_url is not None:
@@ -158,7 +158,7 @@ class RegressorFinder(object):
 
                     commits_to_ignore.append({"rev": backedout, "type": "backedout"})
 
-        logger.info(f"{len(commits_to_ignore)} commits to ignore...")
+        logger.info("%d commits to ignore...", len(commits_to_ignore))
 
         # Skip backed-out commits which aren't in the repository (commits which landed *before* the Mercurial history
         # started, and backouts which mentioned a bad hash in their message).
@@ -166,12 +166,11 @@ class RegressorFinder(object):
             c for c in commits_to_ignore if c["rev"][:12] in all_commits
         ]
 
-        logger.info(f"{len(commits_to_ignore)} commits to ignore...")
+        logger.info("%d commits to ignore...", len(commits_to_ignore))
 
         logger.info(
-            "...of which {} are backed-out".format(
-                sum(1 for commit in commits_to_ignore if commit["type"] == "backedout")
-            )
+            "...of which %d are backed-out",
+            sum(commit["type"] == "backedout" for commit in commits_to_ignore),
         )
 
         db.write(IGNORED_COMMITS_DB, commits_to_ignore)
@@ -194,7 +193,7 @@ class RegressorFinder(object):
             for bug_fixing_commit in db.read(BUG_FIXING_COMMITS_DB)
         )
         logger.info(
-            f"Already classified {len(prev_bug_fixing_commits_nodes)} commits..."
+            "Already classified %d commits...", len(prev_bug_fixing_commits_nodes)
         )
 
         # TODO: Switch to the pure Defect model, as it's better in this case.
@@ -226,7 +225,9 @@ class RegressorFinder(object):
             commit_map[commit["bug_id"]].append(commit["node"])
 
         logger.info(
-            f"{sum(len(commit_list) for commit_list in commit_map.values())} commits found, {len(commit_map)} bugs linked to commits"
+            "%d commits found, %d bugs linked to commits",
+            sum(len(commit_list) for commit_list in commit_map.values()),
+            len(commit_map),
         )
         assert len(commit_map) > 0
 
@@ -235,7 +236,9 @@ class RegressorFinder(object):
 
         bug_count = sum(1 for bug in get_relevant_bugs())
         logger.info(
-            f"{bug_count} bugs in total, {len(commit_map) - bug_count} bugs linked to commits missing"
+            "%d bugs in total, %d bugs linked to commits missing",
+            bug_count,
+            len(commit_map) - bug_count,
         )
 
         known_defect_labels, _ = defect_model.get_labels()
@@ -297,8 +300,8 @@ class RegressorFinder(object):
         db.upload(BUG_FIXING_COMMITS_DB)
 
     def find_bug_introducing_commits(self, repo_dir, tokenized):
-        from pydriller import GitRepository
-        from pydriller.domain.commit import ModificationType
+        # from pydriller import GitRepository
+        # from pydriller.domain.commit import ModificationType
 
         logger.info("Download commits to ignore...")
         assert db.download(IGNORED_COMMITS_DB)
@@ -339,7 +342,7 @@ class RegressorFinder(object):
             for bug_introducing_commit in prev_bug_introducing_commits
         )
         logger.info(
-            f"Already classified {len(prev_bug_introducing_commits)} commits..."
+            "Already classified %d commits...", len(prev_bug_introducing_commits)
         )
 
         hashes_to_ignore = set(commit["rev"] for commit in commits_to_ignore)
@@ -352,7 +355,7 @@ class RegressorFinder(object):
             )
             f.writelines("{}\n".format(git_hash) for git_hash in git_hashes)
 
-        logger.info(f"{len(bug_fixing_commits)} commits to analyze")
+        logger.info("%d commits to analyze", len(bug_fixing_commits))
 
         # Skip already found bug-introducing commits.
         bug_fixing_commits = [
@@ -362,7 +365,8 @@ class RegressorFinder(object):
         ]
 
         logger.info(
-            f"{len(bug_fixing_commits)} commits left to analyze after skipping already analyzed ones"
+            "%d commits left to analyze after skipping already analyzed ones",
+            len(bug_fixing_commits),
         )
 
         bug_fixing_commits = [
@@ -371,7 +375,8 @@ class RegressorFinder(object):
             if bug_fixing_commit["rev"] not in hashes_to_ignore
         ]
         logger.info(
-            f"{len(bug_fixing_commits)} commits left to analyze after skipping the ones in the ignore list"
+            "%d commits left to analyze after skipping the ones in the ignore list",
+            len(bug_fixing_commits),
         )
 
         if tokenized:
@@ -381,138 +386,141 @@ class RegressorFinder(object):
                 if bug_fixing_commit["rev"] in self.mercurial_to_tokenized_git
             ]
             logger.info(
-                f"{len(bug_fixing_commits)} commits left to analyze after skipping the ones with no git hash"
+                "%d commits left to analyze after skipping the ones with no git hash",
+                len(bug_fixing_commits),
             )
 
-        git_init_lock = threading.Lock()
+        # git_init_lock = threading.Lock()
 
-        def _init(git_repo_dir):
-            with git_init_lock:
-                thread_local.git = GitRepository(git_repo_dir)
-                # Call get_head in order to make pydriller initialize the repository.
-                thread_local.git.get_head()
+        # def _init(git_repo_dir):
+        #     with git_init_lock:
+        #         thread_local.git = GitRepository(git_repo_dir)
+        #         # Call get_head in order to make pydriller initialize the repository.
+        #         thread_local.git.get_head()
 
-        def find_bic(bug_fixing_commit):
-            logger.info("Analyzing {}...".format(bug_fixing_commit["rev"]))
+        # def find_bic(bug_fixing_commit):
+        #     logger.info("Analyzing %s...", bug_fixing_commit["rev"])
 
-            git_fix_revision = tuple(mercurial_to_git([bug_fixing_commit["rev"]]))[0]
+        #     git_fix_revision = tuple(mercurial_to_git([bug_fixing_commit["rev"]]))[0]
 
-            commit = thread_local.git.get_commit(git_fix_revision)
+        #     commit = thread_local.git.get_commit(git_fix_revision)
 
-            # Skip huge changes, we'll likely be wrong with them.
-            if len(commit.modifications) > MAX_MODIFICATION_NUMBER:
-                logger.info(
-                    "Skipping {} as it is too big".format(bug_fixing_commit["rev"])
-                )
-                return None
+        #     # Skip huge changes, we'll likely be wrong with them.
+        #     if len(commit.modifications) > MAX_MODIFICATION_NUMBER:
+        #         logger.info(
+        #             "Skipping {} as it is too big".format(bug_fixing_commit["rev"])
+        #         )
+        #         return None
 
-            def get_modification_path(mod):
-                path = mod.new_path
-                if (
-                    mod.change_type == ModificationType.RENAME
-                    or mod.change_type == ModificationType.DELETE
-                ):
-                    path = mod.old_path
-                return path
+        #     def get_modification_path(mod):
+        #         path = mod.new_path
+        #         if (
+        #             mod.change_type == ModificationType.RENAME
+        #             or mod.change_type == ModificationType.DELETE
+        #         ):
+        #             path = mod.old_path
+        #         return path
 
-            bug_introducing_modifications = {}
-            for modification in commit.modifications:
-                path = get_modification_path(modification)
+        #     bug_introducing_modifications = {}
+        #     for modification in commit.modifications:
+        #         path = get_modification_path(modification)
 
-                if path == "testing/web-platform/meta/MANIFEST.json":
-                    continue
+        #         if path == "testing/web-platform/meta/MANIFEST.json":
+        #             continue
 
-                # Don't try to find the bug-introducing commit for modifications
-                # in the bug-fixing commit to non-source code files.
-                if repository.get_type(path) not in repository.SOURCE_CODE_TYPES_TO_EXT:
-                    continue
+        #         # Don't try to find the bug-introducing commit for modifications
+        #         # in the bug-fixing commit to non-source code files.
+        #         if repository.get_type(path) not in repository.SOURCE_CODE_TYPES_TO_EXT:
+        #             continue
 
-                bug_introducing_modifications.update(
-                    thread_local.git.get_commits_last_modified_lines(
-                        commit,
-                        modification=modification,
-                        hashes_to_ignore_path=os.path.realpath("git_hashes_to_ignore"),
-                    )
-                )
+        #         bug_introducing_modifications.update(
+        #             thread_local.git.get_commits_last_modified_lines(
+        #                 commit,
+        #                 modification=modification,
+        #                 hashes_to_ignore_path=os.path.realpath("git_hashes_to_ignore"),
+        #             )
+        #         )
 
-            logger.info(
-                "Found {} for {}".format(
-                    bug_introducing_modifications, bug_fixing_commit["rev"]
-                )
-            )
+        #     logger.info(
+        #         "Found {} for {}".format(
+        #             bug_introducing_modifications, bug_fixing_commit["rev"]
+        #         )
+        #     )
 
-            bug_introducing_commits = []
-            for bug_introducing_hashes in bug_introducing_modifications.values():
-                for bug_introducing_hash in bug_introducing_hashes:
-                    try:
-                        bug_introducing_commits.append(
-                            {
-                                "bug_fixing_rev": bug_fixing_commit["rev"],
-                                "bug_introducing_rev": tuple(
-                                    git_to_mercurial([bug_introducing_hash])
-                                )[0],
-                            }
-                        )
-                    except Exception as e:
-                        # Skip commits that are in git but not in mercurial, as they are too old (older than "Free the lizard").
-                        if not str(e).startswith("Missing git commit in the VCS map"):
-                            raise
+        #     bug_introducing_commits = []
+        #     for bug_introducing_hashes in bug_introducing_modifications.values():
+        #         for bug_introducing_hash in bug_introducing_hashes:
+        #             try:
+        #                 bug_introducing_commits.append(
+        #                     {
+        #                         "bug_fixing_rev": bug_fixing_commit["rev"],
+        #                         "bug_introducing_rev": tuple(
+        #                             git_to_mercurial([bug_introducing_hash])
+        #                         )[0],
+        #                     }
+        #                 )
+        #             except Exception as e:
+        #                 # Skip commits that are in git but not in mercurial, as they are too old (older than "Free the lizard").
+        #                 if not str(e).startswith("Missing git commit in the VCS map"):
+        #                     raise
 
-            # Add an empty result, just so that we don't reanalyze this again.
-            if len(bug_introducing_commits) == 0:
-                bug_introducing_commits.append(
-                    {
-                        "bug_fixing_rev": bug_fixing_commit["rev"],
-                        "bug_introducing_rev": "",
-                    }
-                )
+        #     # Add an empty result, just so that we don't reanalyze this again.
+        #     if len(bug_introducing_commits) == 0:
+        #         bug_introducing_commits.append(
+        #             {
+        #                 "bug_fixing_rev": bug_fixing_commit["rev"],
+        #                 "bug_introducing_rev": "",
+        #             }
+        #         )
 
-            return bug_introducing_commits
+        #     return bug_introducing_commits
 
         def compress_and_upload():
             zstd_compress(db_path)
             db.upload(db_path)
 
-        workers = os.cpu_count() + 1
-        logger.info(
-            f"Analyzing {len(bug_fixing_commits)} commits using {workers} workers..."
-        )
+        # workers = os.cpu_count() + 1
+        # logger.info(
+        #     "Analyzing %d commits using %d workers...",
+        #     len(bug_fixing_commits),
+        #     len(bug_fixing_commits),
+        # )
 
-        with concurrent.futures.ThreadPoolExecutor(
-            initializer=_init, initargs=(repo_dir,), max_workers=workers
-        ) as executor:
+        # with concurrent.futures.ThreadPoolExecutor(
+        #     initializer=_init, initargs=(repo_dir,), max_workers=workers
+        # ) as executor:
 
-            def results():
-                start_time = time.monotonic()
+        #     def results():
+        #         start_time = time.monotonic()
 
-                futures = {
-                    executor.submit(find_bic, bug_fixing_commit): bug_fixing_commit[
-                        "rev"
-                    ]
-                    for bug_fixing_commit in bug_fixing_commits
-                }
+        #         futures = {
+        #             executor.submit(find_bic, bug_fixing_commit): bug_fixing_commit[
+        #                 "rev"
+        #             ]
+        #             for bug_fixing_commit in bug_fixing_commits
+        #         }
 
-                for future in tqdm(
-                    concurrent.futures.as_completed(futures),
-                    total=len(futures),
-                ):
-                    exc = future.exception()
-                    if exc is not None:
-                        logger.info(
-                            f"Exception {exc} while analyzing {futures[future]}"
-                        )
-                        for f in futures:
-                            f.cancel()
+        #         for future in tqdm(
+        #             concurrent.futures.as_completed(futures),
+        #             total=len(futures),
+        #         ):
+        #             exc = future.exception()
+        #             if exc is not None:
+        #                 logger.info(
+        #                     "Exception %s while analyzing %s", exc, futures[future]
+        #                 )
+        #                 for f in futures:
+        #                     f.cancel()
 
-                    result = future.result()
-                    if result is not None:
-                        yield from result
+        #             result = future.result()
+        #             if result is not None:
+        #                 yield from result
 
-                    if time.monotonic() - start_time >= 3600:
-                        compress_and_upload()
-                        start_time = time.monotonic()
+        #             if time.monotonic() - start_time >= 3600:
+        #                 compress_and_upload()
+        #                 start_time = time.monotonic()
 
-            db.append(db_path, results())
+        #     db.append(db_path, results())
 
         compress_and_upload()
 
@@ -534,7 +542,7 @@ def evaluate(bug_introducing_commits):
     for bug in tqdm(bugzilla.get_bugs()):
         if bug["regressed_by"]:
             known_regressors[bug["id"]] = bug["regressed_by"]
-    logger.info(f"Loaded {len(known_regressors)} known regressors")
+    logger.info("Loaded %d known regressors", len(known_regressors))
 
     fix_to_regressors_map = defaultdict(list)
     for bug_introducing_commit in bug_introducing_commits:
@@ -545,9 +553,10 @@ def evaluate(bug_introducing_commits):
             bug_introducing_commit["bug_introducing_rev"]
         )
 
-    logger.info(f"{len(fix_to_regressors_map)} fixes linked to regressors")
+    logger.info("%d fixes linked to regressors", len(fix_to_regressors_map))
     logger.info(
-        f"{sum(len(regressors) for regressors in fix_to_regressors_map.values())} regressors linked to fixes"
+        "%d regressors linked to fixes",
+        sum(len(regressors) for regressors in fix_to_regressors_map.values()),
     )
 
     logger.info("Measuring how many known regressors SZZ was able to find correctly...")
@@ -606,11 +615,11 @@ def evaluate(bug_introducing_commits):
             misassigned_regressors += 1
 
     logger.info(
-        f"Perfectly found {perfect_regressors} regressors out of {all_regressors}"
+        "Perfectly found %d regressors out of %d", perfect_regressors, all_regressors
     )
-    logger.info(f"Found {found_regressors} regressors out of {all_regressors}")
+    logger.info("Found %d regressors out of %d", found_regressors, all_regressors)
     logger.info(
-        f"Misassigned {misassigned_regressors} regressors out of {all_regressors}"
+        "Misassigned %d regressors out of %d", misassigned_regressors, all_regressors
     )
 
 
